/*
 *  ntreg.h
 *  NT Registry driver for Linux
 *
 *  Copyright (c) 1999, Todd Sabin
 */



#ifndef NTREG_H
#define NTREG_H




#define NTREG_SUPER_MAGIC  0x4752544e

#define REGF  0x66676572
#define HBIN  0x6e696268

#define NTREG_IS_REGF(a)          (*(int*)(a)==REGF)
#define NTREG_IS_HBIN(a)          (*(int*)(a)==HBIN)

#define NTREG_BLOCK_SIZE  0x1000


struct ntreg_basic {
        s32 size;
        u16 magic;
};

#define NK_MAGIC (0x6b6e)
#define VK_MAGIC (0x6b76)
#define SK_MAGIC (0x6b73)
#define LF_MAGIC (0x666c)
#define LI_MAGIC (0x696c)
#define RI_MAGIC (0x6972)

#define IS_NK(p)  (((struct ntreg_basic *)(p))->magic == NK_MAGIC)
#define IS_VK(p)  (((struct ntreg_basic *)(p))->magic == VK_MAGIC)
#define IS_SK(p)  (((struct ntreg_basic *)(p))->magic == SK_MAGIC)
#define IS_LF(p)  (((struct ntreg_basic *)(p))->magic == LF_MAGIC)
#define IS_LI(p)  (((struct ntreg_basic *)(p))->magic == LI_MAGIC)
#define IS_RI(p)  (((struct ntreg_basic *)(p))->magic == RI_MAGIC)


/*
 * Nodes entry
 */
struct ntreg_nk {
        s32 node_size;
        u16 magic;
        u16 unk02;
        u32 time1;
        u32 time2;
        u32 hmmm5;
        u32 parent_off;
        u32 num_lf;
        u32 hmmm6;
        u32 lf_off;
        u32 hmmm1;
        u32 num_val;
        u32 val_arr_off;
        u32 sk_off;
        u32 class_off;
        u8 hmmm2[0x14];
        u16 name_len;
        u16 class_len;
        u8 name[1];
};

/*
 * Value entry
 */
struct ntreg_vk {
        s32 node_size;
        u16 magic;
        u16 name_len;
        u32 val_size;
        u32 val_off;
        u32 val_type;
        u8 hmmm2[0x4]; /* seems to always be 01 00 b2 b2 */
        u8 name[1];
};

/*
 * Security descriptor entry
 */
struct ntreg_sk {
        s32 node_size;
        u16 magic;
        u16 sk_unk1;
        u32 sk_prev;
        u32 sk_next;
        u32 sk_links;
        u32 sk_size;
        u8  sk_data[1];
};

/*
 * Subkey (leaf?) entry
 */
struct ntreg_lf {
        s32 node_size;
        u16 magic;
        u16 num_child;
        struct ntreg_lf_entry {
                u32 child_off;
                u8 name[4];
        } u[1];
};

/*
 * Alternate subkey (leaf?) entry
 */
struct ntreg_li {
        s32 node_size;
        u16 magic;
        u16 num_child;
        struct ntreg_li_entry {
                u32 child_off;
        } u[1];
};

/*
 * Another alternate subkey (leaf?) entry
 */
struct ntreg_ri {
        s32 node_size;
        u16 magic;
        u16 num_child;
        struct ntreg_ri_entry {
                u32 child_off;  /* each of these is a pointer to an li
                                   (or lf?) node  */
        } u[1];
};


#define NK_INO(ino)  ((struct ntreg_nk*)((ino)->u.generic_ip))
#define VK_INO(ino)  ((struct ntreg_vk*)((ino)->u.generic_ip))
#define SK_INO(ino)  ((struct ntreg_sk*)((ino)->u.generic_ip))

#define NK(p)        ((struct ntreg_nk*)(p))
#define VK(p)        ((struct ntreg_vk*)(p))
#define SK(p)        ((struct ntreg_sk*)(p))
#define LF(p)        ((struct ntreg_lf*)(p))
#define LI(p)        ((struct ntreg_li*)(p))
#define RI(p)        ((struct ntreg_ri*)(p))

#define VK_DATA_PRESENT(vk)  (((vk)->val_size)&0x80000000)


#endif


/*
 * Local variables:
 *  c-file-style: "linux"
 * End:
 */
